#include <bits/stdc++.h>

using namespace std;

typedef long long ll;

const int MAXN = 1000000;

struct Grana {
    int u, v, root;
    int drugi(int i) { return i ^ u ^ v; }
};

Grana grane[MAXN];
int visited[MAXN], lowlink[MAXN], vreme[MAXN], is_cut[MAXN], koren[MAXN];
int tajmer, broj_komponenti;
vector<int> graf[MAXN], pripada[MAXN];
stack<int> stek;

void izvuci(int e) {
    int r = broj_komponenti++;
    while (true) {
        int e2 = stek.top();
        stek.pop();
        grane[e2].root = r;
        koren[grane[e2].u] = r;
        koren[grane[e2].v] = r;
        pripada[r].push_back(grane[e2].u);
        pripada[r].push_back(grane[e2].v);
        if (e == e2) break;
    }
    sort(pripada[r].begin(), pripada[r].end());
    pripada[r].erase(unique(pripada[r].begin(), pripada[r].end()), pripada[r].end());
}

void dfs_articulation(int node, int parent = -1) {
    lowlink[node] = vreme[node] = tajmer++;
    visited[node] = 1;
    int children = 0;

    for (int e : graf[node]) {
        int next = grane[e].drugi(node);
        if (e == parent || visited[next] == 2) continue;
        stek.push(e);
        if (visited[next]) {
            lowlink[node] = min(lowlink[node], vreme[next]);
        } else {
            children++;
            dfs_articulation(next, e);
            lowlink[node] = min(lowlink[node], lowlink[next]);
            if (lowlink[next] >= vreme[node]) {
                is_cut[node] = 1;
                izvuci(e);
            }
        }
    }
    visited[node] = 2;
    if (parent == -1) is_cut[node] = (children > 1);
}

vector<int> compressed_graph[MAXN];

void priprema(int n) {
    for (int i = 0; i < n; i++) {
        if (!visited[i]) dfs_articulation(i);
    }
    for (int i = 0; i < n; i++) {
        if (is_cut[i]) {
            koren[i] = broj_komponenti++;
            pripada[koren[i]] = {i};
        }
    }
    for (int i = 0; i < n; i++) {
        if (is_cut[i]) {
            for (int e : graf[i]) {
                compressed_graph[grane[e].root].push_back(koren[i]);
                compressed_graph[koren[i]].push_back(grane[e].root);
            }
        }
    }
    for (int i = 0; i < broj_komponenti; i++) {
        sort(compressed_graph[i].begin(), compressed_graph[i].end());
        compressed_graph[i].erase(unique(compressed_graph[i].begin(), compressed_graph[i].end()), compressed_graph[i].end());
    }
}

int a[MAXN];

pair<int, int> find_median(int A, int B) {
    vector<int> values;
    while (A != B) {
        for (int v : pripada[A]) values.push_back(a[v]);
        A = koren[A];
    }
    for (int v : pripada[A]) values.push_back(a[v]);

    sort(values.begin(), values.end());
    int median = values[values.size() / 2];
    return {values.size(), median};
}

int main() {
    ios_base::sync_with_stdio(0);
    cin.tie(0);

    int n, m, q, p;
    cin >> n >> m >> q >> p;

    for (int i = 0; i < n; i++) cin >> a[i];

    for (int i = 0; i < m; i++) {
        int u, v;
        cin >> u >> v;
        u--, v--;
        grane[i] = {u, v};
        graf[u].push_back(i);
        graf[v].push_back(i);
    }

    priprema(n);

    int prev = 0;
    while (q--) {
        int A, B;
        cin >> A >> B;
        A = (A + prev * p) % n;
        B = (B + prev * p) % n;

        if (A == B) {
            cout << a[A] << "\n";
            prev = a[A];
            continue;
        }

        A = koren[A];
        B = koren[B];
        auto [size, median] = find_median(A, B);
        cout << median << "\n";
        prev = median;
    }

    return 0;
}